/*
 * Copyright 2019 Pawel Szczypkowski
 *
 * This file is part of Concul.
 *
 * Concul is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Concul is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Concul.  If not, see <http://www.gnu.org/licenses/>.
 */



#include"crystal.h"
#include<cstdlib>
#include<iostream>
#include<string>
#include<GL/gl.h>
#include<GL/glut.h>
#include<iomanip>
#include<cmath>
#include<vector>
#include"operations.h"
#include<fstream>
#include <allegro5/allegro.h>
#include<allegro5/allegro_native_dialog.h>

 using namespace std;
	
	elements get_symbol(string Symbol){
	
	unsigned i =0;
	while(i<=((sizeof elements_symbols)/(sizeof *elements_symbols)))
	{
	if(elements_symbols[i]==Symbol)
		{
		return static_cast<elements>(i);
		}else
	i++;
	}	
return He;
}

   	Atom::Atom(){
	
		element = He;
		symbol = elements_symbols[(int)element];
		for(int i=0; i<3; i++)
		{
			color[i] = 0;
		}
		x = 0;
		y = 0;
		z = 0;
		radius = atomic_radii[(int)element];
		width = 150;
   	}
	
	Atom::~Atom(){}
	
	Atom::Atom(elements Element, float X, float Y, float Z){

        element=Element;
   	    for(int i=0; i<3; i++)
		{
   	        color[i] = colors_of_atoms[(int)element][i];
		}

		x=X;
		y=Y;
		z=Z;
		radius = atomic_radii[(int)element];
		symbol = elements_symbols[(int)element];
        width = glutStrokeWidth(GLUT_STROKE_ROMAN, symbol[0])+glutStrokeWidth(GLUT_STROKE_ROMAN, symbol[1])+10;
		if((int)element == 5 || (int)element > 108) {
            for (int i = 0; i < 3; i++) {
                letter_color[i] = 1;
            }
        }else{
		    for (int j = 0; j < 3 ; ++j){
		        letter_color[j] = 0;
		    }
		}
	}
	
	Atom::Atom(elements Element,  float X, float Y, float Z, vector<int> Bonding){

        element=Element;
        for(int i=0; i<3; i++)
        {
            color[i] = colors_of_atoms[(int)element][i];
        }

		x=X;
		y=Y;
		z=Z;
		bondingatoms = Bonding;
		radius = atomic_radii[(int)element];
		symbol=elements_symbols[(int)element];
        if((int)element == 5 || (int)element > 108) {
            for (int i = 0; i < 3; i++) {
                letter_color[i] = 1;
            }
        }else{
            for (int j = 0; j < 3 ; ++j){
                letter_color[j] = 0;
            }
        }
        width = 150;
	}
	
	void Atom::add_bonding(vector<int> Bonding){
		bondingatoms = Bonding;
	}
		 	
	void Atom::transparency(){
	
		//this function will make atoms more transparent
	}
	
	void Atom::set_color(float red,float green, float blue){
	
		color[0]=red;
		color[1]=green;
		color[2]=blue;
	}

    void Atom::set_letter_color(float red,float green, float blue){

        letter_color[0]=red;
        letter_color[1]=green;
        letter_color[2]=blue;
    }

	void Atom::set_width(){ //this function has to be called after glut initialization, to aquire symbol width
	    width = glutStrokeWidth(GLUT_STROKE_ROMAN, symbol[0])+glutStrokeWidth(GLUT_STROKE_ROMAN, symbol[1])+10;
	}

	vector<Atom>  Drawing::crystallatice;
	GLfloat Drawing::distance_view;
	float Drawing::RotateMatrix[16];
	float Drawing::InvRotateMatrix[16];
	double Drawing::ProjectionMatrix[16];
	double Drawing::ModelView[16];
	int Drawing::ViewPort[4];
	bool Drawing::new_event = false;
	int Drawing::event;
	float Drawing::camera_x;
    float Drawing::camera_y;
    bool Drawing::if_letters;


	Drawing::Drawing(){
		distance_view = 0.5;
		for(int i=0; i<17; i++)
		{
			RotateMatrix[i] = 0;
			InvRotateMatrix[i] = 0;
		}
		MatrixIdentity(RotateMatrix);
		MatrixIdentity(InvRotateMatrix);
		crystallatice.emplace_back(Atom());
	    camera_x=0;
	    camera_y=0;
	    if_letters = true;
    }
	
	Drawing::~Drawing(){}
	
	Drawing::Drawing(vector<Atom> Crystallatice){
		this->distance_view = 1.5;
		this->crystallatice = Crystallatice;
		MatrixIdentity(RotateMatrix);
		MatrixIdentity(InvRotateMatrix);
		camera_x=0;
		camera_y=0;
		if_letters = true;
    }

	void Drawing::cylinder(Atom A1, Atom A2){
		static double tmpx,tmpy,tmpz;
		tmpx= A2.x-A1.x;
		tmpy= A2.y-A1.y;
		tmpz= A2.z-A1.z;
		
		glRotatef(phi(tmpx,tmpy,tmpz),0,0,1);
		glRotatef(theta(tmpx,tmpy,tmpz),0,1,0);
	
	glEnable(GL_COLOR_MATERIAL);
    glColor3f(A1.color[0],A1.color[1],A1.color[2]);
    
    
    // quadric creation 
    GLUquadricObj * quadcyl = gluNewQuadric();
    
    // way of generation of normal vectors
    gluQuadricNormals( quadcyl, GLU_SMOOTH );
    // orientation of normal vectors
    gluQuadricOrientation( quadcyl, GLU_OUTSIDE );
    // style of quadric
    //gluQuadricDrawStyle( quadobj, GLU_FILL); //GLU_POINT,GLU_LINNE, GLU_SILHOUETTE
    gluCylinder( quadcyl, A1.radius/2, A1.radius/4+A2.radius/4,A1.radius/2-A2.radius/2+sqrt(tmpx*tmpx+tmpy*tmpy+tmpz*tmpz)/2,50, 50);
	
	glDisable(GL_COLOR_MATERIAL);
	
	}
	
	void Drawing::simple_atom(Atom &A){
	
		glPushMatrix();
		glTranslatef(A.x,A.y,A.z);
		
	/*material properties
    //static GLfloat ambient[] = {0.135000, 0.222500, 0.157500, 0.950000};
    //static GLfloat diffuse[] = {0.540000, 0.890000, 0.630000, 0.950000};
    //static GLfloat specular[] = {0.316228, 0.316228, 0.316228, 0.950000};
    //static GLfloat shininess = 12.8;
    //glMaterialfv( GL_FRONT_AND_BACK, GL_AMBIENT, ambient ); 
    //glMaterialfv( GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse );
    //glMaterialfv( GL_FRONT_AND_BACK, GL_SPECULAR, specular );
    //glMaterialf( GL_FRONT_AND_BACK, GL_SHININESS, shininess ); */
    glEnable(GL_COLOR_MATERIAL);
    glColor3f(A.color[0],A.color[1],A.color[2]);
    GLUquadricObj * quadobj = gluNewQuadric();  // quadric creation
    gluQuadricNormals( quadobj, GLU_SMOOTH );   // way of generation of normal vectors
    gluQuadricOrientation( quadobj, GLU_OUTSIDE );  // orientation of normal vectors
    gluSphere( quadobj, A.radius, 50, 50 );
	glPopMatrix();
	unsigned numb = A.bondingatoms.size();

	for(unsigned i=0; i<numb; i++){
	        glPushMatrix();
			glTranslatef(A.x,A.y,A.z);
			cylinder(A, crystallatice.at(A.bondingatoms.at(i)));
			glPopMatrix();
 		}
	//glDisable(GL_COLOR_MATERIAL);
		
	}

    void Drawing::letters(Atom A){

        //simple_atom(A);

        double X1,Y1,Z1,X2,Y2,Z2;
        /* X1,Y1,Z1 are wariables needed to use gluProject in purpose of get the window coordinate of letter*/
        glGetDoublev(GL_MODELVIEW_MATRIX, ModelView);
        glGetDoublev(GL_PROJECTION_MATRIX, ProjectionMatrix);
        glGetIntegerv(GL_VIEWPORT, ViewPort);
        InversingMatrix(RotateMatrix, InvRotateMatrix);

        gluProject(A.x,A.y,A.z,ModelView,ProjectionMatrix,ViewPort,&X1,&Y1,&Z1);
        //gluProject(A.x+InvRotateMatrix[8]*1.3*A.radius,A.y+InvRotateMatrix[9]*A.radius,
                //A.z+InvRotateMatrix[10]*A.radius,ModelView,ProjectionMatrix,ViewPort,&X2,&Y2,&Z2);
        gluProject(A.x+InvRotateMatrix[8]*1.0001*A.radius,A.y+InvRotateMatrix[9]*A.radius,
                   A.z+InvRotateMatrix[10]*A.radius,ModelView,ProjectionMatrix,ViewPort,&X2,&Y2,&Z2);

        gluUnProject(X1,Y1,Z2,ModelView,ProjectionMatrix,ViewPort, &A.letx,&A.lety,&A.letz);

        simple_atom(A);

        glPushMatrix();

        glDisable(GL_LIGHTING);

        glTranslatef(A.letx,A.lety,A.letz);

        glMultMatrixf(InvRotateMatrix);
        glColor3f(A.letter_color[0],A.letter_color[1],A.letter_color[2]);

        glTranslatef(-A.radius/2.,-A.radius/2.,0);
        //float width = glutStrokeWidth(GLUT_STROKE_ROMAN, A.symbol[0])+glutStrokeWidth(GLUT_STROKE_ROMAN, A.symbol[1])+10;
        //A.set_width();      //setting width has to be done here
        glScalef(A.radius/A.width,A.radius/100,1);
        glLineWidth(2.0);

        glutStrokeCharacter(GLUT_STROKE_ROMAN,A.symbol[0]);
        glutStrokeCharacter(GLUT_STROKE_ROMAN,A.symbol[1]);

        glEnable(GL_LIGHTING);

        glPopMatrix();
    }

    void Drawing::draw_everything(){

        glMatrixMode(GL_MODELVIEW);
        glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

        glDisable(GL_LIGHTING);

        axis();
        glLoadIdentity();
        location(distance_view, RotateMatrix,camera_x, camera_y);
        //glGetFloatv(GL_MODELVIEW_MATRIX, ViewMatrix);
        //glRotatef(90,0,0,1);
        glPushMatrix();
        //grid();
        glPopMatrix();

        glEnable(GL_LIGHTING);
        light();

        for(unsigned i=0; i<crystallatice.size(); i++)
        {
            if(if_letters == false) {
                simple_atom(crystallatice.at(i));
            }else
            letters(crystallatice.at(i));
        }
        glutSwapBuffers();
    }

    void Drawing::user_interface(){
	    //press Shift+PPM to rotate
	    //press h to set the home view
	    //from menu: enable letters
	    //getting file from menu

	    //handling shift+PPM to rotate:
        glutKeyboardFunc(keyboard);
        glutIgnoreKeyRepeat(1);
        glutKeyboardUpFunc(key_release);
        glutAttachMenu(GLUT_RIGHT_BUTTON);
        //createmenu();
        //glutMotionFunc(rotation);
        glutMouseFunc(mouse);


       /* glutMouseFunc(display.mouse);

        display.createmenu();
*/

	}

    void Drawing::mouse(int button, int state, int x, int y) {

        switch (button) {
            case 3:
                distance_view -= distance_view*0.05;
                break;
            case 4:
                distance_view +=distance_view*0.05;
                break;
            case GLUT_RIGHT_BUTTON:
                if (state == GLUT_DOWN) {
                    if(event!=Z_ROTATION) {
                        event = ROTATION;
                    }
                    new_event = true;
                    glutMotionFunc(mouse_movements);
                } else
                    event = DISABLE_EVERYTHING;
                break;
            case GLUT_LEFT_BUTTON:
                if (state == GLUT_DOWN) {
                    event = MOVING;
                    new_event = true;
                    glutMotionFunc(mouse_movements);
                } else
                    event = DISABLE_EVERYTHING;
                break;
        }
    }

	void Drawing::mouse_movements(int x, int y){
	    if(event == ROTATION){
	        rotation(x,y);
	    } else if(event == MOVING){
	        move_scene(x,y);
	    } else if(event == Z_ROTATION){
	        zaxis_rotation(x,y);
	    }
	}

	void Drawing::rotation(int x, int y){

            static float temp_x = x;  //coursor x coordinate one moment earlier
            static float temp_y = y;    //coursor y coordinate one moment earlier
            static float dx;        //x displacement
            static float dy;        //y displacement

            if (new_event)  //when we notice that button was pressed in this moment
            {
                temp_x = x;
                temp_y = y;
                new_event = false;
            }
            dx = temp_x - x;
            dy = temp_y - y;

            glPushMatrix();                                                //creating new matrics which will rotate our scene around the dx,dy,0
            glLoadIdentity();                                            //temporary axis by an angle
            glRotatef(-(AbsValue(dx) + AbsValue(dy)) / 1.8, dy, dx, 0);        // this 1.8 const is optimalized value
            //for rotation speed (the greater the value, the slower rotation is)
            glMultMatrixf(RotateMatrix);                                //multiplying matrics to aply small rotation
            glGetFloatv(GL_MODELVIEW_MATRIX, RotateMatrix);                //getting new values to rotation matrics
            glPopMatrix();
            temp_x = x;
            temp_y = y;
    }

	void Drawing::zaxis_rotation(int x, int y){

	    static float temp_x=x;
		static float temp_y=y;
		static float dx;
		static float dy;
        static int width = glutGet(GLUT_WINDOW_WIDTH)/2;
        static int height = glutGet(GLUT_WINDOW_HEIGHT)/2;

	if(new_event){

	    temp_x = x;
		temp_y = y;
		new_event = false;
	} 
	dx = temp_x - x;
	dy = temp_y - y;
	    glPushMatrix();
		glLoadIdentity();
		if(absolut(dx,dy)){   					//wich move is greater
			if(x < width){		//where is the mouse I&III
				glRotatef(-dy/120.0,0, 0, 1);
				}else{
			glRotatef(dy/120.0,0, 0, 1);
           }	
		}else if(y < height){
			glRotatef(dx/120.0,0, 0, 1);
			 }else{
		glRotatef(-dx/120.0,0, 0, 1);
		}		  
		
		glMultMatrixf(RotateMatrix);
		glGetFloatv(GL_MODELVIEW_MATRIX, RotateMatrix);
		glPopMatrix();
	dx = temp_x - x;
	dy = temp_y - y;

}

    void Drawing::move_scene(int x, int y){
            static float temp_x = x;  //coursor x coordinate one moment earlier
            static float temp_y = y;    //coursor y coordinate one moment earlier
            static float dx;        //x displacement
            static float dy;        //y displacement
            if (new_event)  //when we notice that button was pressed in this moment
            {
                temp_x = x;
                temp_y = y;
                new_event = false;
            }
            dx = temp_x - x;
            dy = temp_y - y;
            camera_x -=dx*distance_view/1000;
            camera_y +=dy*distance_view/1000;

            temp_x = x;
            temp_y = y;
	}

	void Drawing::keyboard(unsigned char c, int x, int y) {
        static bool fog = false;
        static float fog_color[] = {0.8, 0.8, 0.8, 1.0};
        switch (c) {

            case 'z':
                new_event = true;
                event = Z_ROTATION;
                glutMotionFunc(mouse_movements);
                break;
            case 'r':
                new_event = true;
                glutDetachMenu(GLUT_RIGHT_BUTTON);
                event = ROTATION;
                glutMotionFunc(mouse_movements);
            break;
            case 'R':
                new_event = true;
                glutDetachMenu(GLUT_RIGHT_BUTTON);
                event = ROTATION;
                glutMotionFunc(mouse_movements);
            break;
            case 'h':
                home();
                break;
            case 'H':
                home();
                break;
            case 'l':
                crystallatice.clear();
                loadfile(&crystallatice,file_dialog_box());
                break;
            case 's':
                if(if_letters == true){
                    if_letters = false;
                }else
                    if_letters = true;
                break;
            case 'f':
                if(!fog){
                    fog = true;
                    glEnable(GL_FOG);
                    glFogi(GL_FOG_MODE, GL_EXP);
                    glFogf(GL_FOG_DENSITY, 0.02);
                    glFogfv(GL_FOG_COLOR, fog_color);

                }else{
                    fog = false;
                    glDisable(GL_FOG);
                }
        }
    }

    void Drawing::key_release(unsigned char key, int xx, int yy){

	    switch (key) {
            case 'r':
                glutAttachMenu(GLUT_RIGHT_BUTTON);
                event = DISABLE_EVERYTHING;
            break;
            case 'R':
                glutAttachMenu(GLUT_RIGHT_BUTTON);
                event = DISABLE_EVERYTHING;
            break;
            case 'z':
                event = DISABLE_EVERYTHING;
        }

	}

	void Drawing::home(){
	    //distance_view = 1.0;

	    vector<float> edges = get_home_edges(crystallatice);
        float cube_dimx = (edges.at(1)-edges.at(0))/2.;
        float cube_dimy = (edges.at(3)-edges.at(2))/2.;
        float cube_dimz = (edges.at(5)-edges.at(4))/2.;
	    double height = 1.0*glutGet(GLUT_WINDOW_HEIGHT)-5;
        double width = 1.0*glutGet(GLUT_WINDOW_WIDTH)-5;
        double X1,Y1,Z1;
        /* X1,Y1,Z1 are wariables needed to use gluProject in purpose of get the window coordinate of letter*/

        for (int i = 0; i < 500; ++i) {

            glGetDoublev(GL_MODELVIEW_MATRIX, ModelView);
            glGetDoublev(GL_PROJECTION_MATRIX, ProjectionMatrix);
            glGetIntegerv(GL_VIEWPORT, ViewPort);
            gluProject(cube_dimx, cube_dimy, cube_dimz, ModelView, ProjectionMatrix, ViewPort, &X1, &Y1, &Z1);
            if (X1 < width && Y1 < height && 0<=Y1 && 0<=X1) {
                break;
            }
            distance_view += 0.05*distance_view;

            draw_everything();
        }
	}

    void Drawing::createmenu(){
        glutCreateMenu(mymenu);
        glutAddMenuEntry("Chose File",1);
        glutAddMenuEntry("Sth else",2);
        glutAddMenuEntry("+",3);
        glutAddMenuEntry("-",4);
        // attach the menu to the right button
        glutAttachMenu(GLUT_RIGHT_BUTTON);
    }

    void Drawing::mymenu(int option){
        switch (option){
            case 1:
                crystallatice.clear();

                loadfile(&crystallatice, file_dialog_box());
                break;
            case 2:
                crystallatice.clear();
                crystallatice.push_back(Atom(C,0,0,0));
                break;
            case 3:
                crystallatice.at(0).set_letter_color(crystallatice.at(0).letter_color[0]+0.1,crystallatice.at(0).letter_color[1]+0.1,crystallatice.at(0).letter_color[2]+0.1);
                break;
            case 4:
                crystallatice.at(0).set_letter_color(crystallatice.at(0).letter_color[0]-0.1,crystallatice.at(0).letter_color[1]-0.1,crystallatice.at(0).letter_color[2]-0.1);
        }
        event = 6;
    }

    void Drawing::axis(){

        double X1,Y1,Z1;

        glPushMatrix();
        glLoadIdentity();

        glGetDoublev(GL_MODELVIEW_MATRIX, ModelView);
        glGetDoublev(GL_PROJECTION_MATRIX, ProjectionMatrix);
        glGetIntegerv(GL_VIEWPORT, ViewPort);
        gluUnProject(25,45,0,ModelView,ProjectionMatrix,ViewPort, &X1,&Y1,&Z1);

        //glTranslatef(-0.89,0.54,-1.5);
        glTranslatef(X1,-Y1,-1.5);
        location(0.1, RotateMatrix,0,0);

        glColor3f(1,0,0);  //red
        drawarrow();  //x axis

        glRotatef(90, 0,0,1);
        glColor3f(0,1,0);		//green
        drawarrow(); /*y axis*/

        glRotatef(-90,0,1,0);
        glColor3f(0,0,1);  //blue
        drawarrow(); //z axis

        glPopMatrix();


}

    void bonding(vector<Atom> *crystallatice){
        vector<int> bondinglist;
        unsigned n = crystallatice->size();
        for(unsigned i=0; i< n; i++) {
            for (unsigned j = 0; j < n; j++) {
                if (two_atoms_bonding(crystallatice->at(j), crystallatice->at(i))) {
                    bondinglist.push_back(j);
                }
            }
            (crystallatice->at(i)).add_bonding(bondinglist);
            bondinglist.clear();
        }
    }

    void loadfile(vector<Atom> *Crystallatice, string path){
        //opening filedialog window and choosing a file
        //string path = file_dialog_box();
        //string path = "periodictable.xyz";

        fstream file;
        file.open(path.c_str(), ios::in);   //only reading data from file
        if(file.good())
        {
            string line;
            while(!file.eof())
            {
                getline(file, line);
                float x,y,z;
                string Symbol;
                istringstream iss(line);
                iss>>Symbol>>x>>y>>z;
                Crystallatice->push_back(Atom(get_symbol(Symbol),x,y,z));
            }
            file.close();
        }// here I'll put a function which is going to write error window
        bonding(Crystallatice);
    }

    string file_dialog_box(){

        if (!al_init()) //initialization of allegro library
        {
            cout << "failed to initialize allegro" << endl;
        }

        ALLEGRO_FILECHOOSER *filechooser;                //variable for creating a file dialog box

        filechooser = al_create_native_file_dialog(NULL, "Chose a File", "*.xyz;*.txt",
                                               ALLEGRO_FILECHOOSER_FILE_MUST_EXIST);
        //now the *.txt and .xyz files can be choosen
        if (al_show_native_file_dialog(NULL, filechooser))    //showing displayed dialog box
        {
            string path = al_get_native_file_dialog_path(filechooser, 0);
            al_destroy_native_file_dialog(filechooser);
            return path;  //returns the path
        }
        return "ERROR";
    }

    bool two_atoms_bonding(Atom A1, Atom A2){
        double r;
        r = (A1.x-A2.x)*(A1.x-A2.x)+(A1.y-A2.y)*(A1.y-A2.y)+(A1.z-A2.z)*(A1.z-A2.z);

        if(sqrt(r) <= (A1.radius+A2.radius))
        {
            return true;
        }else
            return false;
    }

    vector<float> get_home_edges(vector<Atom> crystallatice){

	    int size = crystallatice.size();
	    vector<float> edges;
        float min[3];
	    float max[3];
	    if(size == 0){
            for (int j = 0; j < 3 ; ++j) {
                edges.push_back(1);
                edges.push_back(-1);
            }
	        return edges;
	    }
        min[0] = crystallatice.at(0).x-crystallatice.at(0).radius;
        max[0] = crystallatice.at(0).x+crystallatice.at(0).radius;
        min[1] = crystallatice.at(0).y-crystallatice.at(0).radius;
        max[1] = crystallatice.at(0).y+crystallatice.at(0).radius;
        min[2] = crystallatice.at(0).z-crystallatice.at(0).radius;
        max[2] = crystallatice.at(0).z+crystallatice.at(0).radius;
        //min and max x
        for (int i = 0; i < size ; ++i) {
            if(min[0]<-100){
                min[0] = -99;
                break;
            }
            if(min[0]> crystallatice.at(i).x-crystallatice.at(i).radius){
                min[0] =crystallatice.at(i).x - crystallatice.at(i).radius;
            }
        }
        for (int j = 0; j < size ; ++j) {
            if(max[0]>100){
                max[0] = 99;
                break;
            }
            if(max[0]< crystallatice.at(j).x+crystallatice.at(j).radius){
                max[0] =crystallatice.at(j).x+crystallatice.at(j).radius;
            }
        }
        //min and max y
        for (int i = 0; i < size ; ++i) {
            if(min[1]<-100){
                min[1]= -99;
                break;
            }else if(min[1]> crystallatice.at(i).y-crystallatice.at(i).radius){
                min[1] =crystallatice.at(i).y-crystallatice.at(i).radius;
            }
        }
        for (int j = 0; j < size ; ++j) {
            if(max[1]>100){
                max[1] = 99;
                break;
            }
            if(max[1]< crystallatice.at(j).y+crystallatice.at(j).radius){
                max[1] =crystallatice.at(j).y+crystallatice.at(j).radius;
            }
        }
        //min and max z
        for (int i = 0; i < size ; ++i) {
            if(min[2]<-100){
                min[2]= -99;
                break;
            }else if(min[2]> crystallatice.at(i).z-crystallatice.at(i).radius){
                min[2] =crystallatice.at(i).z-crystallatice.at(i).radius;
            }
        }
        for (int j = 0; j < size ; ++j) {
            if(max[2]>100){
                max[2] = 99;
                break;
            }
            if(max[2]< crystallatice.at(j).z+crystallatice.at(j).radius){
                max[2] =crystallatice.at(j).z+crystallatice.at(j).radius;
            }
        }
        for (int j = 0; j < 3 ; ++j) {
            edges.push_back(min[j]);
            edges.push_back(max[j]);
        }
        return edges;
    }

/*static GLfloat zbuffer[2][2];
 glReadPixels(x,y,2,2,GL_DEPTH_COMPONENT,GL_FLOAT,zbuffer);
 //cout<<distance_view<<" "<<zbuffer[1][1]<<" "<<crystallatice.at(0).z_buffer<<" ";
 cout<<x_prim(crystallatice.at(0).x,crystallatice.at(0).y,crystallatice.at(0).z,InvRotateMatrix)<<" ";
 cout<<y_prim(crystallatice.at(0).x,crystallatice.at(0).y,crystallatice.at(0).z,InvRotateMatrix)<<" " ;
 cout<<z_prim(crystallatice.at(0).x,crystallatice.at(0).y,crystallatice.at(0).z,InvRotateMatrix)<<endl;
*/

/*	void Drawing::roboczy(){

	glMatrixMode(GL_MODELVIEW);
		glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

	glPushMatrix();
glLineWidth(2.0);

		location(distance_view, RotateMatrix);

	  glDisable(GL_LIGHTING);
	  glColor3f(1,0,0);  //red
	  drawarrow();  //x axis

	  glRotatef(90, 0,0,1);
	  glColor3f(0,1,0);		//green
	  drawarrow(); y axis

	  glRotatef(-90,0,1,0);
	  glColor3f(0,0,1);  //blue
	  drawarrow(); //z axis

	glPopMatrix();
glEnable(GL_LIGHTING);
		light();
letters(crystallatice.at(2));
	glutSwapBuffers();
	}

void Drawing::letters1(Atom A){


	A.z_buffer = (10-1/(distance_view-z_prim(A.x,A.y,A.x,InvRotateMatrix)-A.radius))/9.99;
	if(A.z_buffer > 1){
	cout<<"distance view= "<<distance_view<<" z_prim="<<InvRotateMatrix[8]*A.x+InvRotateMatrix[9]*A.y+InvRotateMatrix[10]*A.z<<" wiekszy od 1"<<endl;
			A.z_buffer = 0.9944;
	}else if(A.z_buffer <= 0){
		A.z_buffer = 0.4;
		cout<<"distance view= "<<distance_view<<" z_prim="<<InvRotateMatrix[8]*A.x+InvRotateMatrix[9]*A.y+InvRotateMatrix[10]*A.z<<" mniejszy od 0 "<<endl;;
	}
	//	cout<<InvRotateMatrix[8]*A.x+InvRotateMatrix[9]*A.y+InvRotateMatrix[10]*A.z;


	//cout<<A.z_buffer<<endl;

	glPushMatrix();
   	glTranslatef(A.x, A.y, A.z);

      InversingMatrix(RotateMatrix, InvRotateMatrix);
	  glMultMatrixf(InvRotateMatrix);

		glEnable(GL_COLOR_MATERIAL);
		glColor3f(A.color[0],A.color[1],A.color[2]);
		GLUquadricObj * quadobj = gluNewQuadric();
		gluQuadricNormals( quadobj, GLU_SMOOTH );
        gluQuadricOrientation( quadobj, GLU_OUTSIDE );

        glDepthRange (0, 1);
        glDepthFunc(GL_LESS);
        //glPolygonOffset(-0,600000);


        gluSphere( quadobj, A.radius, 50, 50 );

		//glDisable(GL_POLYGON_OFFSET_LINE);

		glDisable(GL_COLOR_MATERIAL);
		if(distance_view<1.0 &&A.z_buffer>0.03){
		glDepthRange(A.z_buffer-0.023, A.z_buffer-0.02);
	}else if(distance_view > 1.0 && A.z_buffer>0.03) {
		 glDepthRange(A.z_buffer-0.023/distance_view, A.z_buffer-0.03/distance_view);
	 }else if (A.z_buffer<0.03){
		 glDepthRange(0.01,0.02);
	 }
		glDepthFunc(GL_LEQUAL);
		glDisable(GL_LIGHTING);
		//glPoligonMode();

		//glEnable(GL_POLYGON_OFFSET_LINE);

		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
		glutKeyboardFunc(Drawing::keyboard);
		glBegin(GL_POLYGON);
		 glColor3f(0,1,0);
		 glVertex3f(-A.radius/2,A.radius/2,0);
		 glVertex3f(A.radius/2,A.radius/2,0);
		 glVertex3f(A.radius/2,-A.radius/2,0);
		 glVertex3f(-A.radius/2,-A.radius/2,0);
		 glEnd();
		static float x_p,y_p, z_p;
		x_p = x_prim(A.x,A.y,A.z, RotateMatrix);
		y_p = y_prim(A.x,A.y,A.z, RotateMatrix);
		z_p = z_prim(A.x,A.y,A.z, RotateMatrix);
		glRotatef(x_angle(x_p,y_p,z_p,distance_view),1,0,0);
		glRotatef(y_angle(x_p,y_p,z_p,distance_view),1,0,0);
// cout<<x_p<<" "<<y_p<<" "<<z_p<<" "<<distance_view<<" "<<x_angle(x_p,y_p,z_p,distance_view)<<" "<<y_angle(x_p,y_p,z_p,distance_view)<<endl;

 glColor3f(1,0,0);  //red
 drawarrow();  //x axis

 glRotatef(90, 0,0,1);
 glColor3f(0,1,0);		//green
 drawarrow(); //y axis

 glRotatef(-90,0,1,0);
 glColor3f(0,0,1);  //blue
 drawarrow(); //z axis

   glColor3f(0,0,0);
   static int symbol_length = A.symbol.length();

static float width = 0;
   if(width==0){
   for(int i=0; i<=symbol_length; i++)
   {
       width += glutStrokeWidth(GLUT_STROKE_ROMAN, A.symbol[i]);
   }
}
   glTranslatef(-A.radius*1.41/2.,-A.radius*1.41/2.,A.radius);
   glScalef(A.radius/width,A.radius/100,1);
   glLineWidth(2.0);

   glutStrokeCharacter(GLUT_STROKE_ROMAN,A.symbol[0]);
   glutStrokeCharacter(GLUT_STROKE_ROMAN,A.symbol[1]);


   glColor3f(0,0,0);
   static int symbol_length = A.symbol.length();

static float width = 0;
   if(width==0){
   for(int i=0; i<=symbol_length; i++)
   {
       width += glutStrokeWidth(GLUT_STROKE_ROMAN, A.symbol[i]);
   }
}
   glTranslatef(-A.radius/2,-A.radius/2.,0);
   glScalef(A.radius/width,A.radius/100,1);
   glLineWidth(2.0);

   glutStrokeCharacter(GLUT_STROKE_ROMAN,A.symbol[0]);
   glutStrokeCharacter(GLUT_STROKE_ROMAN,A.symbol[1]);

glDepthRange (0, 1);
glEnable(GL_LIGHTING);

glPopMatrix();

}
*/



